package com.xkrs.dao;

import com.xkrs.model.entity.Nationwide;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author XinYi Song
 */
@Component
public interface NationwideDao extends JpaRepository<Nationwide,Long> {

    /**
     * 通过省编码查询市名称和编码
     * @param proCode
     * @return
     */
    @Query(value = "select DISTINCT city_name,city_code from nationwide where pro_code = ?",nativeQuery = true)
    List<Map<String,String>> findCityByProCode(String proCode);

    /**
     * 通过省编码查询县名称和编码
     * @param proCode
     * @return
     */
    @Query(value = "select county_name,county_code from nationwide where pro_code = ?",nativeQuery = true)
    List<Map<String,String>> findCountyByProCode(String proCode);

    /**
     * 通过市编码查询区县名称和编码
     * @param cityCode
     * @return
     */
    @Query(value = "select county_name,county_code from nationwide where city_code = ?",nativeQuery = true)
    List<Map<String,String>> findCountyByCityCode(String cityCode);
}
