package com.xkrs.utils;

import com.xkrs.dao.FirePointDao;
import com.xkrs.model.entity.FirePointEntity;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author XinYi Song
 */

@Component
public class Query {

    @Resource
    private FirePointDao firePointDao;

    /**
     * 动态多条件查询项目信息(省)
     */
    public List<FirePointEntity> selectFirePointByPro(String proName, String cityCode, String satelliteType, String landType, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (proName != null && !"".equals(proName)) {
                list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + proName + "%"));
            }
            if (cityCode != null && !"".equals(cityCode)) {
                list.add(criteriaBuilder.like(root.get("countyCode").as(String.class), "%" + cityCode + "%"));
            }
            if (satelliteType != null && !"".equals(satelliteType)) {
                list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
            }
            if (landType != null && !"".equals(landType)) {
                list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            LocalDateTime localDateTime = LocalDateTime.now();
            LocalDateTime localDateTime2 = localDateTime.plusDays(1);
            String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
            // 当前时间减30
            LocalDateTime localDateTime1 = localDateTime.minusDays(30);
            String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
            list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 动态多条件查询项目信息(市)
     */
    public List<FirePointEntity> selectFirePointByCity(String cityName, String countyCode, String satelliteType, String landType, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (cityName != null && !"".equals(cityName)) {
                list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + cityName + "%"));
            }
            if (countyCode != null && !"".equals(countyCode)) {
                list.add(criteriaBuilder.like(root.get("countyCode").as(String.class), "%" + countyCode + "%"));
            }
            if (satelliteType != null && !"".equals(satelliteType)) {
                list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
            }
            if (landType != null && !"".equals(landType)) {
                list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            LocalDateTime localDateTime = LocalDateTime.now();
            LocalDateTime localDateTime2 = localDateTime.plusDays(1);
            String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
            // 当前时间减30
            LocalDateTime localDateTime1 = localDateTime.minusDays(30);
            String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
            list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 动态多条件查询项目信息(县)
     */
    public List<FirePointEntity> selectFirePointByCounty(String countyCode, String streetCode, String satelliteType, String landType, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (countyCode != null && !"".equals(countyCode)) {
                list.add(criteriaBuilder.like(root.get("countyCode").as(String.class), "%" + countyCode + "%"));
            }
            if (streetCode != null && !"".equals(streetCode)) {
                list.add(criteriaBuilder.like(root.get("streetCode").as(String.class), "%" + streetCode + "%"));
            }
            if (satelliteType != null && !"".equals(satelliteType)) {
                list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
            }
            if (landType != null && !"".equals(landType)) {
                list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            LocalDateTime localDateTime = LocalDateTime.now();
            LocalDateTime localDateTime2 = localDateTime.plusDays(1);
            String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
            // 当前时间减30
            LocalDateTime localDateTime1 = localDateTime.minusDays(30);
            String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
            list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 动态多条件查询项目信息(街道)
     */
    public List<FirePointEntity> selectFirePointByStreet(String streetCode, String satelliteType, String landType, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (streetCode != null && !"".equals(streetCode)) {
                list.add(criteriaBuilder.like(root.get("streetCode").as(String.class), streetCode));
            }
            if (satelliteType != null && !"".equals(satelliteType)) {
                list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
            }
            if (landType != null && !"".equals(landType)) {
                list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            LocalDateTime localDateTime = LocalDateTime.now();
            LocalDateTime localDateTime2 = localDateTime.plusDays(1);
            String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
            // 当前时间减30
            LocalDateTime localDateTime1 = localDateTime.minusDays(30);
            String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
            list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一周的火点
     */
    public List<FirePointEntity> selectFirePointBetweenThree(String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点信息（省）
     */
    public List<FirePointEntity> selectFirePointByMonthPro(String address, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (address != null && !"".equals(address)) {
                list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + address + "%"));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点信息（市）
     */
    public List<FirePointEntity> selectFirePointByMonthCity(String address, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (address != null && !"".equals(address)) {
                list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + address + "%"));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点信息（县）
     */
    public List<FirePointEntity> selectFirePointByMonthCounty(String countyCode, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (countyCode != null && !"".equals(countyCode)) {
                list.add(criteriaBuilder.like(root.get("countyCode").as(String.class), countyCode));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点信息（街道）
     */
    public List<FirePointEntity> selectFirePointByMonthStreet(String streetCode, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (streetCode != null && !"".equals(streetCode)) {
                list.add(criteriaBuilder.like(root.get("streetCode").as(String.class), streetCode));
            }
            if (startTime != null && !"".equals(startTime)) {
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
            }
            if (endTime != null && !"".equals(endTime)) {
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
            }
            Predicate[] predicates = new Predicate[list.size()];
            return criteriaBuilder.and(list.toArray(predicates));
        };
        return firePointDao.findAll(specification);
    }


}